#!/usr/bin/perl

use strict;

my $VERSION = 0.2;
my $PROGRAM = 'ch4_args.pl';

my $debug = 0;
for my $test (@ARGV)
{
    if ($test =~ /--help/)
    {
	&print_help;
	exit;
    }
    if ($test =~/--version/)
    {
	print "$PROGRAM is: $VERSION\n";
	exit;
    }
    if ($test =~ /--debug/)
    {
	$debug = 1;
    }
}

my @arg = @ARGV;
my %input = &get_input(@arg);
my $f = 0.9999;

my $Ar = defined($input{'Ar'}) ? $input{'Ar'}*$f : 0;
my $CO2 = defined($input{'CO2'}) ? $input{'CO2'}*$f : 0;
my $CO = defined($input{'CO'}) ? $input{'CO'}*$f : 0;
my $CH4 = defined($input{'CH4'}) ? $input{'CH4'}*$f : 0.3333;
my $O2 = defined($input{'O2'}) ? $input{'O2'}*$f : 0;
my $H2 = defined($input{'H2'}) ? $input{'H2'}*$f : 0;
my $H2O = defined($input{'H2O'}) ? $input{'H2O'}*$f : 0.6666;
my $temp = defined($input{'cell'}) ? $input{'cell'} : 850;
my $atm = defined($input{'P'}) ? $input{'P'} : 1; 
my $calc = 'Ar';
#my $calc = defined($input{'calc'}) ? $input{'calc'} : 'Ar';
my $inputstr = "\nInput was:\nAr: $Ar\nCO2: $CO2\nCO: $CO\nCH4 $CH4\nO2 $O2\nH2 $H2\nH2O $H2O\ncalculated on basis of $calc\n";
#my $cl = 2;

#my @gas = ('Ar','CO2','CO','CH4','O2','H2','H2O');
#for (my $x = 0 ; $x < scalar(@gas); $x++)
#{
#    if ($calc =~ /^$gas[$x]$/)
#    {
#	$cl = $x + 2;
#    }
#} 
my $ch4calc = '/usr/local/bin/ch4_arg';

if (($H2 < 0 ) && ($CO > -$H2))
{
    ### Corrects input data from OCV_corr ###
    ### for calculations with high fuel   ###
    ### utilisation                       ###
    $CO = $CO + $H2;
    $H2O = $H2O + $H2;
    $CO2 = $CO2 - $H2;
    $H2 = 0;
}

#### Calls ch4_args ###
my $cmd = $ch4calc." --h2=$H2 --h2o=$H2O --o2=$O2 --ch4=$CH4 --co2=$CO2 --co=$CO --celsius=$temp --pres=$atm";
my @out = `$cmd`;
print @out if ($debug);


if (!defined($input{'command'}))
{
    print $inputstr;
    print @out;
    exit;
}

my $command = $input{'command'};
$out[@out-1] =~ /([\d\-\.]+)\s+mv/;
my $OCV = $1;
$out[@out-2] =~ /([\d\-\.]+)\s+mv/;
my $OCV_O2 = $1;
$out[@out-3] =~ /([\d\-\.]+)[\s\t]+$/;
my $a_C = $1;

my %partP = ();
$out[@out-5] =~ /([\d\-\.\+Ee]+)[\s\t]+$/;
$partP{'O2'} = $1+0;
$out[@out-6] =~ /([\d\-\.\+Ee]+)[\s\t]+$/;
$partP{'CO'} = $1+0;
$out[@out-7] =~ /([\d\-\.\+Ee]+)[\s\t]+$/;
$partP{'CO2'} = $1+0;
$out[@out-8] =~ /([\d\-\.\+Ee]+)[\s\t]+$/;
$partP{'CH4'} = $1+0;
$out[@out-9] =~ /([\d\-\.\+Ee]+)[\s\t]+$/;
$partP{'H2O'} = $1+0;
$out[@out-10] =~ /([\d\-\.\+Ee]+)[\s\t]+$/;
$partP{'H2'} = $1+0;


if ($command =~ /^OCV$/i)
{
    print "$OCV\n";
}
if ($command =~ /^OCV_O2$/i)
{
    print "$OCV_O2\n";
}
if ($command =~ /^a_C$/i)
{
    print "$a_C\n";
}
if ($command =~ /^par_all$/i)
{
    print "emf_vs_air  $OCV\n";
    print "emf_vs_O2   $OCV_O2\n";
    print "Carbon_act  $a_C\n";
    foreach my $key (sort keys %partP)
    {
	print "Part_P_$key  $partP{$key}\n";
    }
}
if ($command =~ /^all$/i)
{
    print $inputstr;
    print @out;
}
exit;



sub get_input
{
    my %out = ();
    my @input = @_;
    for (my $x = 0; $x < scalar(@input);$x++)
    {
        if ($input[$x] =~ /^*--/)
        {
            $input[$x] =~ s/^*--//;
            $input[$x+1] =~ s/^*[\s]+//;
	    $out{$input[$x]} = $input[$x+1];
            $x++;
        }
    }
    return %out;
}


sub print_help
{
    print "\n\nch4.pl is a program using the pascal program made by Bjoern Malmgren\nIt calculates the theoretical cell voltage (emf) of a gas mixture of:\n CH_4, CO, CO_2, H_2, O_2 and H_2O\n";
    print "\n  Usage of ch4.pl:\n";
    print "ch4.pl [opt: --version --help --debug] [opt command \$cmd] --KEY1 \$key1 --KEY2 \$key2 ..\n\n";
    print "  \$cmd is the command options:\n";
    print "  all       : Verbose output, prints all information (same as no command).\n";
    print "  OCV       : Prints only the theoretical emp of the gas.\n\n";
    print "  OCV_O2    : Prints the theoretical emf of the gas versus pure oxygen.\n";
    print "  par_all   : Prints all parameters in formatted form: Key Value\n";
    print "              one line at a time.\n";
    print "  a_C       : Prints the carbon activity.\n";
    print "The --KEY \$key values specify the gasses and other parameters:\n";
    print "  --cell    : Inputs the gas temperature, default is 850C.\n";
    print "  --P       : Inputs the gas pressure, default is 1 atm.\n";
    print "  --O2      : Inputs the oxygen (O2 \!) mole fraction.\n";
    print "  --CH4     : Inputs the methane mole fraction.\n";
    print "  --CO      : Inputs the carbon monoxide mole fraction.\n";
    print "  --CO2     : Inputs the carbon dioxide fraction.\n";
    print "  --H2      : Inputs the hydrogen (H2 \!) mole fraction.\n";
    print "  --H2O     : Inputs the water mole fraction.\n";
    print "  --Ar      : Inputs the mole fraction of Argon (or an other inert gas).\n\n";
    print "If the mole fractions of the gasses does not add up to 1 then the Argon\nmole fraction is addjusted accordingly.\nRemember to do check that the mole fractions add up!!\nIf the mole fractions adds to above 1 unpredictable results may occour!\n\n";
    print "Coding and design by Soeren Koch.\n\n";
}
